package com.gic.haoban.manage.web.controller.order;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.business.order.dto.ordermanage.ListOrderDTO;
import com.gic.business.order.dto.ordermanage.OrderDetailDTO;
import com.gic.business.order.enums.ActivityPreferentialTypeEnum;
import com.gic.business.order.enums.OrderEnum;
import com.gic.business.order.qdto.ordermanage.ListOrderManageQDTO;
import com.gic.business.order.qdto.ordermanage.OrderItemShipmentQDTO;
import com.gic.business.order.qdto.ordermanage.OrderShipmentQDTO;
import com.gic.business.order.service.ordermanage.WebOrderManageApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.haoban.manage.web.controller.marketing.MemberMarketingController;
import com.gic.haoban.manage.web.vo.order.OrderDetailVO;
import com.gic.haoban.manage.web.vo.order.OrderListItemVO;
import com.gic.haoban.manage.web.vo.order.OrderListVO;

@RestController
@RequestMapping("/order")
public class GicOrderController {

    private static final org.apache.logging.log4j.Logger logger = LogManager.getLogger(MemberMarketingController.class);

    @Autowired
    private WebOrderManageApiService webOrderManageApiService;

    // 发货订单列表 (待发货+待收货)
    @RequestMapping("list-order")
    public RestResponse<Object> orderList(String enterpriseId, String storeId, String clerkId, String search,
            BasePageInfo pageInfo) {
        // 查询门店
        ListOrderManageQDTO qdto = new ListOrderManageQDTO();
        qdto.setOrderShow(1);
        qdto.setOrderStep(9);
        qdto.setBusinessType(OrderEnum.BusinessTypeEnum.MICRO_MALL.getType());
        qdto.setSearchOrderParam(search);
        qdto.setDeliveryType(-1);
        qdto.setEnterpriseId(enterpriseId);
        qdto.setPageSizeZero(false);
        qdto.setCount(false);
        qdto.setPageNum(pageInfo.getPageNum());
        qdto.setPageSize(pageInfo.getPageSize());
        ServiceResponse<Page<ListOrderDTO>> ret = webOrderManageApiService.listOrder(qdto);
        logger.info("订单信息={}", JSON.toJSONString(ret, true));
        if (ret.isSuccess()) {
            Page<OrderListVO> retPage = PageHelperUtils.changePageToCurrentPage(ret.getResult(), OrderListVO.class);
            return RestResponse.successResult(retPage);
        }
        return RestResponse.failure(ret.getCode(), ret.getMessage());
    }

    // 数量查询(待发货+待收货)
    // orderType:1待发货 2待收货
    public RestResponse<Object> orderCount(String enterpriseId, String clerkId, String searchParams,
            @RequestParam(defaultValue = "1") int orderType) {
        int count = 0;
        return RestResponse.successResult(count);
    }

    // 订单详情 (客户信息+收货地址+商品详情+订单信息 + 订单状态)
    @RequestMapping("order-detail")
    public RestResponse<Object> orderDetail(String enterpriseId, String clerkId, String searchParams,
            @RequestParam(defaultValue = "1") int orderType, String orderId) {
        ServiceResponse<OrderDetailDTO> ret = webOrderManageApiService.getOrderDetail(enterpriseId, orderId);
        if (!ret.isSuccess()) {
            return RestResponse.failure(ret.getCode(), ret.getMessage());
        }
        logger.info("订单信息={}", JSON.toJSONString(ret, true));
        OrderDetailVO orderDetailVO = EntityUtil.changeEntityByJSON(OrderDetailVO.class, ret.getResult());
        String discountAmountInfo = "{}";
        JSONObject jsonObject = JSONObject.parseObject(discountAmountInfo);
        // 单品
        BigDecimal discountActivity = jsonObject
                .getBigDecimal(ActivityPreferentialTypeEnum.ActivityEnum.DISCOUNT_ACTIVITY.getType().toString());
        discountActivity = (discountActivity != null ? discountActivity : BigDecimal.ZERO);
        // 等级
        BigDecimal memberGrade = jsonObject
                .getBigDecimal(ActivityPreferentialTypeEnum.ActivityEnum.MEMBER_GRADE.getType().toString());
        memberGrade = (memberGrade != null ? memberGrade : BigDecimal.ZERO);
        // 满减
        BigDecimal fullReduce = jsonObject
                .getBigDecimal(ActivityPreferentialTypeEnum.ActivityEnum.FULL_REDUCTION.getType().toString());
        fullReduce = (fullReduce != null ? fullReduce : BigDecimal.ZERO);

        // 抵金券
        BigDecimal cashCoupon = jsonObject
                .getBigDecimal(ActivityPreferentialTypeEnum.ActivityEnum.CASH_COUPON.getType().toString());
        cashCoupon = (cashCoupon != null ? cashCoupon : BigDecimal.ZERO);

        // 折扣券
        BigDecimal discountCoupons = jsonObject
                .getBigDecimal(ActivityPreferentialTypeEnum.ActivityEnum.DISCOUNT_COUPONS.getType().toString());
        discountCoupons = (discountCoupons != null ? discountCoupons : BigDecimal.ZERO);
        return RestResponse.successResult(orderDetailVO);
    }

    // 订单发货商品列表
    @RequestMapping("order-logistics-deliver")
    public RestResponse<Object> orderDeliverList(String enterpriseId, String orderId) {
        ServiceResponse<OrderDetailDTO> resp = this.webOrderManageApiService.getOrderDetail(enterpriseId, orderId);
        List<OrderListItemVO> list = null;
        return RestResponse.successResult(list);
    }

    // 发货
    @RequestMapping("order-logistics-deliver")
    public RestResponse<Object> orderDeliver(String enterpriseId, String orderId, String storeId,
            List<String> orderItemIdList, String companyCode, String logisticsNo, String companyId,
            String companyName) {
        OrderShipmentQDTO qdto = new OrderShipmentQDTO();
        qdto.setEnterpriseId(enterpriseId);
        qdto.setLogisticsCompanyCode(companyCode);
        qdto.setLogisticsCompanyName(companyName);
        qdto.setLogisticsCompanyId(companyId);
        qdto.setCourierNumber(logisticsNo);
        qdto.setOrderId(orderId);
        // 门店发货
        qdto.setDeliveryMode(2);
        // 好办
        qdto.setSource(2);
        List<OrderItemShipmentQDTO> list = new ArrayList<>();
        orderItemIdList.forEach(o -> {
            OrderItemShipmentQDTO item = new OrderItemShipmentQDTO();
            item.setOrderItemId(o);
            item.setStoreId(storeId);
            list.add(item);
        });
        qdto.setOrderItemShipment(list);
        ServiceResponse<Boolean> response = webOrderManageApiService.orderShipment(qdto);
        if (!response.isSuccess()) {
            return RestResponse.failure(response.getCode(), response.getMessage());
        }
        return RestResponse.successResult(response.getResult());
    }

    // 修改物流
    @RequestMapping("modify-order-logistics")
    public RestResponse<Object> modifyOrderDeliver(String enterpriseId, String orderId, String storeId,
            List<String> orderItemIdList, String companyCode, String logisticsNo, String companyId,
            String companyName) {
        OrderShipmentQDTO qdto = new OrderShipmentQDTO();
        qdto.setEnterpriseId(enterpriseId);
        qdto.setLogisticsCompanyCode(companyCode);
        qdto.setLogisticsCompanyName(companyName);
        qdto.setLogisticsCompanyId(companyId);
        qdto.setCourierNumber(logisticsNo);
        qdto.setOrderId(orderId);
        // 门店发货
        qdto.setDeliveryMode(2);
        // 好办
        qdto.setSource(2);
        List<OrderItemShipmentQDTO> list = new ArrayList<>();
        orderItemIdList.forEach(o -> {
            OrderItemShipmentQDTO item = new OrderItemShipmentQDTO();
            item.setOrderItemId(o);
            item.setStoreId(storeId);
            list.add(item);
        });
        qdto.setOrderItemShipment(list);
        this.webOrderManageApiService.updateLogistics(qdto);
        return RestResponse.successResult();
    }
    // 客户详情中的商城订单
}
